#include "whetstone.h"

void sleep(uint16_t msecs);
void chkkey();
float readtemp();

void setup() {
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.begin();

  __HAL_RCC_DTS_CLK_ENABLE();
  sleep(1);

  //pclk is running at 250 Mhz, internal freq must be < 1 Mhz for calibration
  //set HSREF_CLK_DIV to 0x7f (127)
  DTS->CFGR1 |= DTS_CFGR1_HSREF_CLK_DIV_Msk;
  //enable digital temperature sensor  
  DTS->CFGR1 |= DTS_CFGR1_TS1_EN;
  // improve accuracy by using 5 periods
  DTS->CFGR1 |= 5 << DTS_CFGR1_TS1_SMP_TIME_Pos;

}

void loop() {
  Serial.println("Hello stm32h562");
  sleep(1000);
  whetstone(0);
  float t = readtemp();
  Serial.print("Temp: ");
  Serial.println(t);
  chkkey();
  sleep(10000);
}

void sleep(uint16_t msecs) {
  for(uint16_t i=0; i<msecs;i++)
    asm("wfi");
}

float readtemp() {
  // wait for TS1_RDY
  while(! DTS->SR & DTS_SR_TS1_RDY) sleep(1);
  // clear data register
  DTS->DR = 0;
  // start measuremennt
  DTS->CFGR1 |= DTS_CFGR1_TS1_START;
  // wait for TS1_RDY
  while(! DTS->SR & DTS_SR_TS1_RDY) sleep(1);
  // read the measurement value
  int32_t ts1_mfreq = DTS->DR & 0xfffful;
  
  /* from ref manual
  T = T_0 + (( F_PCLK ⁄ TS1_MFREQ ) * TS1_SMP_TIME - 100 * TS1_FMT0 ) / TS1_RAMP_COEFF
  */
  float t0 = 30.0;
  float f_pclk = 250e6;
  int32_t ts1_fmt0 = DTS->T0VALR1 & 0xfffful;
  int32_t ts1_ramp_coeff = DTS->RAMPVALR & 0xfffful;
  int16_t ts1_smp_time = 5;

  Serial.print("ts1_mfreq ");
  Serial.println(ts1_mfreq);

  Serial.print("ts1_fmt0 ");
  Serial.println(ts1_fmt0);

  Serial.print("ts1_ramp_coeff ");
  Serial.println(ts1_ramp_coeff);

  float t = t0 + (( f_pclk / ts1_mfreq) * ts1_smp_time - 100 * ts1_fmt0) / ts1_ramp_coeff;

  return t;

}

void chkkey() {
  if (Serial.available()) {
    while(Serial.available()) Serial.read();
    Serial.println("pause");
    while(true) {
      sleep(1000);
      if(Serial.available()) break;      
    }
    while(Serial.available()) Serial.read();
    Serial.println("resume");
  }
}

void blinks() {
  for(uint8_t i=0; i<5; i++) {
    digitalToggle(LED_BUILTIN);
    sleep(100);
    digitalToggle(LED_BUILTIN);
    sleep(200);
  }
}